==> BestCrypt-2.0-0.tar.gz: bcrypt-2.0-0/src/bctool-link.cpp <==
#include "bctool.h"
#include "container/file.h"

#include <unistd.h>

bool BCTool::DoLink(const char *file, BCVolume *volume, char *out_deviceName, bool readOnly)
{
    char tmpDeviceName[BC_MAX_PATH] = {0};
    bc_kern_result kernelResult;
    if (!out_deviceName)
        out_deviceName = tmpDeviceName;
    int options = readOnly ? kBCDisk_ReadOnly : 0;
    kernelResult = BCKernelSetupDisk(file, volume->info.alg, volume->info.mode, volume->key_handle,
                              volume->pool, volume->info.map, options, out_deviceName);

    if (kBCKernel_Success != kernelResult)
    {
        this->ShowError("Could not link container file %s to device. Error while creating block device.\n", file);
        return false;
    }

    // Send BLKRRPART ioctl to our device for kernel to rescan it's partition table.
    if (this->RunProcessL("/sbin/bcrrpart_helper", out_deviceName, ARG_TERM))
    {
        this->ShowWarning("Could not rescan partitions on device. BLKRRPART ioctl failed.\n");
    }

    return true;
}

bool BCTool::Link(const char *file, bool readOnly, BCMountParams *mountParams, bool mainPartOnly,
                  BCVolume *out_volume, BCPassword *out_password, char *out_deviceName)
{
    // Check file name.
    if (strlen(file) == 0)
    {
        this->ShowError("File name cannot be empty.\n");
        return false;
    }

    // Check file.
    if (!BCTool::WantRead(file))
        return false;

    // Check write permissions if requested.
    if(!CanWrite(file) && !readOnly)
    {
        this->ShowWarning("You do not have write permissions for container file %s. Will link for read only.\n",
                          file);
        readOnly = true;
    }

    if (this->CheckContainerInUse(file))
    {
        return false;
    }

    BCVolume tmpVolume;
    if (!out_volume)
        out_volume = &tmpVolume;
    int error = BCDecodeVolume(file, NULL, mainPartOnly, mountParams, out_password, out_volume);
    if (error != kBCContainer_Success)
    {
        if (error != kBCContainer_Cancelled && error != kBCContainer_InvalidPassword)
        {
            this->ShowError("Could not parse container file %s: %s\n", file,
                            BCContainerErrorDecription((BCContainerResult)error));
        }
        return false;
    }

    // Check container file size to prevent backup mounting.
    if ((!out_volume->info.hidden) && (BCFileSize(file) < (out_volume->info.offset + out_volume->info.size)))
    {
        this->ShowError("File size is less than the size specified in a container header. Possibly you are trying to mount a container header backup.\n");
        return false;
    }

    bool result = this->DoLink(file, out_volume, out_deviceName, readOnly);

    if (out_volume == &tmpVolume)
        BCPutVolume(out_volume);

    return result;
}

bool BCTool::Unlink(const char *deviceName, bool force)
{
    if (!BCTool::IsBlockSpecial(deviceName))
    {
        this->ShowError("%s is not a block device.\n", deviceName);
        return false;
    }

    int error = BCKernelRemoveDisk(deviceName, force? 1 : 0);

    if (error)
    {
        this->ShowError("Could not remove device %s: %s\n", deviceName, BCKernelGetErrorString(error));
        return false;
    }

    return true;
}

==> BestCrypt-2.0-4.tar.gz: bcrypt-2.0-4/src/bctool-link.cpp <==
#include "bctool.h"
#include "container/file.h"

#include <unistd.h>

bool BCTool::DoLink(const char *file, BCVolume *volume, char *out_deviceName, bool readOnly)
{
    char tmpDeviceName[BC_MAX_PATH] = {0};
    bc_kern_result kernelResult;
    if (!out_deviceName)
        out_deviceName = tmpDeviceName;
    int options = readOnly ? kBCDisk_ReadOnly : 0;
    kernelResult = BCKernelSetupDisk(file, volume->info.alg, volume->info.mode, volume->key_handle,
                              volume->pool, volume->info.map, options, out_deviceName);

    if (kBCKernel_Success != kernelResult)
    {
        this->ShowError("Could not link container file %s to device. Error while creating block device.\n", file);
        return false;
    }

    // Send BLKRRPART ioctl to our device for kernel to rescan it's partition table.
    if (this->RunProcessL("/sbin/bcrrpart_helper", out_deviceName, ARG_TERM))
    {
        this->ShowWarning("Could not rescan partitions on device. BLKRRPART ioctl failed.\n");
    }

    return true;
}

bool BCTool::Link(const char *file, bool readOnly, BCMountParams *mountParams, bool mainPartOnly,
                  BCVolume *out_volume, BCPassword *out_password, char *out_deviceName)
{
    // Check file name.
    if (strlen(file) == 0)
    {
        this->ShowError("File name cannot be empty.\n");
        return false;
    }

    // Check file.
    if (!BCTool::WantRead(file))
        return false;

    // Check write permissions if requested.
    if(!CanWrite(file) && !readOnly)
    {
        this->ShowWarning("You do not have write permissions for container file %s. Will link for read only.\n",
                          file);
        readOnly = true;
    }

    if (this->CheckContainerInUse(file))
    {
        return false;
    }

    BCVolume tmpVolume;
    if (!out_volume)
        out_volume = &tmpVolume;
    int error = BCDecodeVolume(file, NULL, mainPartOnly, mountParams, out_password, out_volume);
    if (error != kBCContainer_Success)
    {
        if (error != kBCContainer_Cancelled && error != kBCContainer_InvalidPassword)
        {
            this->ShowError("Could not parse container file %s: %s\n", file,
                            BCContainerErrorDecription((BCContainerResult)error));
        }
        return false;
    }

    // Check container file size to prevent backup mounting.
    if ((!out_volume->info.hidden)
            && (!BCTool::IsBlockSpecial(file)) // ignore condition for block devices.
            && (
                (BCFileSize(file) != out_volume->info.size) // for containers created with bcrypt 1.8-0, the container size is equal to the file size.
                && (BCFileSize(file) < (out_volume->info.offset + out_volume->info.size))
               ))
    {
        this->ShowError("File size is less than the size specified in a container header. Possibly you are trying to mount a container header backup.\n");
        return false;
    }
    else if (BCTool::IsBlockSpecial(file)
             && (out_volume->info.size == BCFileSize(file))
             && (!out_volume->info.hidden))
    {
        // Deal with bug in previous versions of BestCrypt here.
        // Version 1.9-2 stored a whole size of the block device in header file,
        // without excluding of the header size, so we should modify the value
        // before sending it to a driver.

        // Bug results in IO error when dealing with last 8 blocks.
        //
        // The exact solution should:
        // 1) Write new, right size to the container header.
        // 2) Explain how to deal with block device that changed it's size.
        //
        // The solution below simply fixes the size of the container before
        // sending it to the kernel.

        // Old versions had kgSHA as default keygen, so we can remove only 4096KB.
        out_volume->info.size -= 4096;
        sm_result smr;
        SpaceMap *modifiedMap;

        smr = SpaceMapGetMapOfSize(out_volume->info.map, out_volume->info.size, &modifiedMap);
        if (kSpaceMap_Success != smr)
        {
            this->ShowError("Could not reallocate space map while linking: %s. Please contact our technical support.\n", SpaceMapGetErrorString(smr));
            return false;
        }

        SpaceMapFree(out_volume->info.map);
        out_volume->info.map = modifiedMap;
    }

    bool result = this->DoLink(file, out_volume, out_deviceName, readOnly);

    if (out_volume == &tmpVolume)
        BCPutVolume(out_volume);

    return result;
}

bool BCTool::Unlink(const char *deviceName, bool force)
{
    if (!BCTool::IsBlockSpecial(deviceName))
    {
        this->ShowError("%s is not a block device.\n", deviceName);
        return false;
    }

    int error = BCKernelRemoveDisk(deviceName, force? 1 : 0);

    if (error)
    {
        this->ShowError("Could not remove device %s: %s\n", deviceName, BCKernelGetErrorString(error));
        return false;
    }

    return true;
}
